@@ -19,7 +19,7 @@
     Boston, MA 02110-1301, USA.
 */
 
-#include "vcardtool.h"
+#include "vcardtool_p.h"
 #include "key.h"
 #include "picture.h"
 #include "secrecy.h"
@@ -30,9 +30,6 @@
 #include "fieldgroup.h"
 #include "kcontacts_debug.h"
 #include <QString>
-#include <QBuffer>
-#include <QDebug>
-#include <QRegularExpression>
 
 using namespace KContacts;
 
@@ -243,8 +240,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list, VCard::Version v
             if (version == VCard::v4_0) {
                 Geo geo = (*it).geo();
                 if (geo.isValid()) {
-                    QString str;
-                    str.sprintf("\"geo:%.6f,%.6f\"", geo.latitude(), geo.longitude());
+                    QString str = QString::asprintf("\"geo:%.6f,%.6f\"", geo.latitude(), geo.longitude());
                     adrLine.addParameter(QStringLiteral("GEO"), str);
                 }
             }
@@ -293,7 +289,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list, VCard::Version v
             card.addLine(line);
         }
 
-        const Related::List relatedList = (*addrIt).relationShips();
+        const Related::List relatedList = (*addrIt).relationships();
         Related::List::ConstIterator relatedIt;
         Related::List::ConstIterator relatedEnd(relatedList.end());
         for (relatedIt = relatedList.begin(); relatedIt != relatedEnd; ++relatedIt) {
@@ -373,9 +369,9 @@ QByteArray VCardTool::createVCards(const Addressee::List &list, VCard::Version v
         if (geo.isValid()) {
             QString str;
             if (version == VCard::v4_0) {
-                str.sprintf("geo:%.6f,%.6f", geo.latitude(), geo.longitude());
+                str = QString::asprintf("geo:%.6f,%.6f", geo.latitude(), geo.longitude());
             } else {
-                str.sprintf("%.6f;%.6f", geo.latitude(), geo.longitude());
+                str = QString::asprintf("%.6f;%.6f", geo.latitude(), geo.longitude());
             }
             card.addLine(VCardLine(QStringLiteral("GEO"), str));
         }
@@ -569,14 +565,13 @@ QByteArray VCardTool::createVCards(const Addressee::List &list, VCard::Version v
         // TODO Add vcard4.0 support
         const TimeZone timeZone = (*addrIt).timeZone();
         if (timeZone.isValid()) {
-            QString str;
 
             int neg = 1;
             if (timeZone.offset() < 0) {
                 neg = -1;
             }
 
-            str.sprintf("%c%02d:%02d", (timeZone.offset() >= 0 ? '+' : '-'),
+            QString str = QString::asprintf("%c%02d:%02d", (timeZone.offset() >= 0 ? '+' : '-'),
                         (timeZone.offset() / 60) * neg,
                         (timeZone.offset() % 60) * neg);
 
@@ -652,8 +647,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list, VCard::Version v
         // IMPP (supported in vcard 3 too)
         const QVector<Impp> lstImpp = (*addrIt).imppList();
         for (const Impp &impp : lstImpp) {
-            VCardLine line(QStringLiteral("IMPP"), impp.address());
-            line.addParameter(QStringLiteral("X-SERVICE-TYPE"), Impp::typeToString(impp.type()));
+            VCardLine line(QStringLiteral("IMPP"), impp.address().url());
             QMapIterator<QString, QStringList> i(impp.parameters());
             while (i.hasNext()) {
                 i.next();
@@ -873,22 +867,13 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                 }
                 //IMPP
                 else if (identifier == QLatin1String("impp")) {
-                    QString imppStr = (*lineIt).value().toString();
+                    QUrl imppUrl((*lineIt).value().toString());
                     Impp impp;
                     impp.setParameters((*lineIt).parameterMap());
-                    if (!(*lineIt).parameter(QStringLiteral("x-service-type")).isEmpty()) {
-                        impp.setAddress(imppStr);
-                        const QString serviceType = (*lineIt).parameter(QStringLiteral("x-service-type")).toLower();
-                        imppService(serviceType, impp);
-                    } else {
-                        const int pos = imppStr.indexOf(QLatin1Char(':'));
-                        if (pos != -1) {
-                            const QString serviceType = imppStr.left(pos);
-                            const QString address = imppStr.right(imppStr.length() - pos - 1);
-                            impp.setAddress(address);
-                            imppService(serviceType, impp);
-                        }
+                    if (!(*lineIt).parameter(QStringLiteral("x-service-type")).isEmpty() && imppUrl.scheme().isEmpty()) {
+                        imppUrl.setScheme(normalizeImppServiceType((*lineIt).parameter(QStringLiteral("x-service-type")).toLower()));
                     }
+                    impp.setAddress(imppUrl);
                     addr.insertImpp(impp);
                 }
                 // CLASS
@@ -1155,7 +1140,7 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                     Related related;
                     related.setRelated((*lineIt).value().toString());
                     related.setParameters((*lineIt).parameterMap());
-                    addr.insertRelationShip(related);
+                    addr.insertRelationship(related);
                 }
                 // CLIENTPIDMAP (vcard 4.0)
                 else if (identifier == QLatin1String("clientpidmap")) {
@@ -1169,20 +1154,30 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                 else if (identifier.startsWith(QLatin1String("x-"))) {
                     QString ident = (*lineIt).identifier();
                     //X-Evolution
+                    // also normalize case of our own extensions, some backends "adjust" that
                     if (identifier == QLatin1String("x-evolution-spouse")
                         || identifier == QLatin1String("x-spouse")) {
                         ident = QStringLiteral("X-KADDRESSBOOK-X-SpousesName");
-                    } else if (identifier == QLatin1String("x-evolution-blog-url")) {
+                    } else if (identifier == QLatin1String("x-evolution-blog-url") || identifier.compare(QLatin1String("X-KADDRESSBOOK-BLOGFEED"), Qt::CaseInsensitive) == 0) {
                         ident = QStringLiteral("X-KADDRESSBOOK-BlogFeed");
                     } else if (identifier == QLatin1String("x-evolution-assistant")
-                               || identifier == QLatin1String("x-assistant")) {
+                               || identifier == QLatin1String("x-assistant")
+                               || identifier.compare(QLatin1String("X-KADDRESSBOOK-X-ASSISTANTSNAME"), Qt::CaseInsensitive) == 0) {
                         ident = QStringLiteral("X-KADDRESSBOOK-X-AssistantsName");
                     } else if (identifier == QLatin1String("x-evolution-anniversary")
-                               || identifier == QLatin1String("x-anniversary")) {
+                               || identifier == QLatin1String("x-anniversary")
+                               || identifier.compare(QLatin1String("X-KADDRESSBOOK-X-ANNIVERSARY"), Qt::CaseInsensitive) == 0) {
                         ident = QStringLiteral("X-KADDRESSBOOK-X-Anniversary");
                     } else if (identifier == QLatin1String("x-evolution-manager")
-                               || identifier == QLatin1String("x-manager")) {
+                               || identifier == QLatin1String("x-manager")
+                               || identifier.compare(QLatin1String("X-KADDRESSBOOK-X-MANAGERSNAME"), Qt::CaseInsensitive) == 0) {
                         ident = QStringLiteral("X-KADDRESSBOOK-X-ManagersName");
+                    } else if (identifier.compare(QLatin1String("X-KADDRESSBOOK-X-PROFESSION"), Qt::CaseInsensitive) == 0) {
+                        ident = QStringLiteral("X-KADDRESSBOOK-X-Profession");
+                    } else if (identifier.compare(QLatin1String("X-KADDRESSBOOK-X-OFFICE"), Qt::CaseInsensitive) == 0) {
+                        ident = QStringLiteral("X-KADDRESSBOOK-X-Office");
+                    } else if (identifier.compare(QLatin1String("X-KADDRESSBOOK-X-SPOUSESNAME"), Qt::CaseInsensitive) == 0) {
+                        ident = QStringLiteral("X-KADDRESSBOOK-X-SpousesName");
                     } else if (identifier == QLatin1String("x-aim")) {
                         ident = QStringLiteral("X-messaging/aim-All");
                     } else if (identifier == QLatin1String("x-icq")) {
@@ -1215,8 +1210,22 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
 
                     const QString key = ident.mid(2);
                     const int dash = key.indexOf(QLatin1Char('-'));
-                    addr.insertCustom(key.left(dash), key.mid(dash + 1),
-                                      (*lineIt).value().toString());
+
+                    // convert legacy messaging fields into IMPP ones
+                    if (key.startsWith(QLatin1String("messaging/"))) {
+                        QUrl url;
+                        url.setScheme(normalizeImppServiceType(key.mid(10, dash - 10)));
+                        const auto values = (*lineIt).value().toString().split(QChar(0xE000), QString::SkipEmptyParts);
+                        for (const auto &value : values) {
+                            url.setPath(value);
+                            Impp impp;
+                            impp.setParameters((*lineIt).parameterMap());
+                            impp.setAddress(url);
+                            addr.insertImpp(impp);
+                        }
+                    } else {
+                        addr.insertCustom(key.left(dash), key.mid(dash + 1), (*lineIt).value().toString());
+                    }
                 }
             }
         }
@@ -1581,37 +1590,16 @@ QStringList VCardTool::splitString(QChar sep, const QString &str) const
     return list;
 }
 
-void VCardTool::imppService(const QString &serviceType, KContacts::Impp &impp) const
+QString VCardTool::normalizeImppServiceType(const QString &serviceType) const
 {
-    if (serviceType == QLatin1String("facebook")) {
-        impp.setType(KContacts::Impp::Facebook);
-    } else if (serviceType == QLatin1String("jabber")) {
-        impp.setType(KContacts::Impp::Jabber);
-    } else if (serviceType == QLatin1String("sip")) {
-        impp.setType(KContacts::Impp::Sip);
-    } else if (serviceType == QLatin1String("aim")) {
-        impp.setType(KContacts::Impp::Aim);
-    } else if (serviceType == QLatin1String("msn")) {
-        impp.setType(KContacts::Impp::Msn);
-    } else if (serviceType == QLatin1String("twitter")) {
-        impp.setType(KContacts::Impp::Twitter);
-    } else if (serviceType == QLatin1String("googletalk")) {
-        impp.setType(KContacts::Impp::GoogleTalk);
-    } else if (serviceType == QLatin1String("xmpp")) {
-        impp.setType(KContacts::Impp::Xmpp);
-    } else if (serviceType == QLatin1String("icq")) {
-        impp.setType(KContacts::Impp::Icq);
-    } else if (serviceType == QLatin1String("yahoo")) {
-        impp.setType(KContacts::Impp::Yahoo);
-    } else if (serviceType == QLatin1String("qq")) {
-        impp.setType(KContacts::Impp::Qq);
-    } else if (serviceType == QLatin1String("gadugadu")) {
-        impp.setType(KContacts::Impp::GaduGadu);
-    } else if (serviceType == QLatin1String("owncloud-handle")) {
-        impp.setType(KContacts::Impp::Ownclound);
-    } else if (serviceType == QLatin1String("skype")) {
-        impp.setType(KContacts::Impp::Skype);
-    } else {
-        qCDebug(KCONTACTS_LOG) << "unknown service type " << serviceType;
+    if (serviceType == QLatin1String("jabber")) {
+        return QStringLiteral("xmpp");
     }
+    if (serviceType == QLatin1String("yahoo")) {
+        return QStringLiteral("ymsgr");
+    }
+    if (serviceType == QLatin1String("gadugadu")) {
+        return QStringLiteral("gg");
+    }
+    return serviceType;
 }
